@@ -1,23 +1,28 @@
 import asyncio
+import json
+import sys
+import tempfile
 import time
 import traceback
 from collections.abc import Coroutine, Sequence
+from pathlib import Path
 from typing import TYPE_CHECKING, Any, Callable, Literal, Unpack, overload
 
+# import websockets
 from halo import Halo  # pyright: ignore[reportMissingTypeStubs]
-from loguru import logger
 from notte_core.agent_types import AgentCompletion
+from notte_core.common.logging import logger
 from notte_core.common.notifier import BaseNotifier
 from notte_core.common.telemetry import track_usage
 from notte_core.utils.webp_replay import MP4Replay, WebpReplay
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, Field, ValidationError
 from typing_extensions import final
-from websockets.asyncio import client
 
 from notte_sdk.endpoints.base import BaseClient, NotteEndpoint
 from notte_sdk.endpoints.personas import NottePersona
 from notte_sdk.endpoints.sessions import RemoteSession
 from notte_sdk.endpoints.vaults import NotteVault
+from notte_sdk.endpoints.workflows import RemoteWorkflow
 from notte_sdk.types import (
     AgentCreateRequest,
     AgentCreateRequestDict,
@@ -29,10 +34,25 @@ from notte_sdk.types import (
     AgentStatus,
     AgentStatusRequest,
     AgentStatusResponse,
+    AgentWorkflowCodeRequest,
+    AgentWorkflowCodeResponse,
+    GetWorkflowResponse,
     SdkAgentCreateRequest,
     SdkAgentStartRequestDict,
 )
 
+# Conditional imports for Pyodide vs native Python
+RUNNING_IN_PYODIDE = "pyodide" in sys.modules
+
+if RUNNING_IN_PYODIDE:
+    import js  # pyright: ignore[reportMissingImports]
+    from pyodide.ffi import (  # pyright: ignore [reportMissingImports]
+        create_proxy,  # pyright: ignore [reportUnknownVariableType]
+    )
+else:
+    from websockets.asyncio import client
+
+
 if TYPE_CHECKING:
     from notte_sdk.client import NotteClient
 
@@ -48,7 +68,7 @@ class LegacyAgentStatusResponse(AgentStatusResponse):
     This is a temporary solution to avoid breaking changes.
     """
 
-    steps: list[dict[str, Any]] = Field(default_factory=list)  # pyright: ignore[reportIncompatibleVariableOverride]
+    steps: list[dict[str, Any]] = Field(default_factory=list)
 
 
 @final
@@ -65,8 +85,9 @@ class AgentsClient(BaseClient):
     AGENT_START_CUSTOM = "start/custom"
     AGENT_STOP = "{agent_id}/stop?session_id={session_id}"
     AGENT_STATUS = "{agent_id}"
+    AGENT_WORKFLOW = "{agent_id}/workflow/code"
     AGENT_LIST = ""
-    # The following endpoints downloads a .webp file
+    # The following endpoints downloads a MP4 file
     AGENT_REPLAY = "{agent_id}/replay"
     AGENT_LOGS_WS = "{agent_id}/debug/logs?token={token}&session_id={session_id}"
 
@@ -149,6 +170,24 @@ class AgentsClient(BaseClient):
             path = path.format(agent_id=agent_id)
         return NotteEndpoint(path=path, response=LegacyAgentStatusResponse, method="GET")
 
+    @staticmethod
+    def _agent_workflow_endpoint(agent_id: str | None = None) -> NotteEndpoint[AgentWorkflowCodeResponse]:
+        """
+        Creates an endpoint for retrieving an agent's script.
+
+        If an agent ID is provided, formats the endpoint path to target that specific agent.
+
+        Args:
+            agent_id: Optional identifier of the agent; if specified, the endpoint path will include this ID.
+
+        Returns:
+            NotteEndpoint configured with the GET method and AgentWorkflowCodeResponse as the expected response.
+        """
+        path = AgentsClient.AGENT_WORKFLOW
+        if agent_id is not None:
+            path = path.format(agent_id=agent_id)
+        return NotteEndpoint(path=path, response=AgentWorkflowCodeResponse, method="GET")
+
     @staticmethod
     def _agent_replay_endpoint(agent_id: str | None = None) -> NotteEndpoint[BaseModel]:
         """
@@ -249,66 +288,157 @@ class AgentsClient(BaseClient):
 
         async def get_messages() -> AgentStatusResponse | None:
             counter = 0
-            async with client.connect(
-                uri=wss_url,
-                open_timeout=30,
-                ping_interval=5,
-                ping_timeout=40,
-                close_timeout=5,
-            ) as websocket:
-                try:
-                    async for message in websocket:
-                        assert isinstance(message, str), f"Expected str, got {type(message)}"
-                        try:
-                            if agent_id in message and "agent_id" in message:
-                                # termination condition
-                                response_message = AgentStatusResponse.model_validate_json(message)
-                                if not response_message.success:
-                                    logger.error(response_message.answer)
-                                return response_message
-                            response = AgentCompletion.model_validate_json(message)
-                            if log:
-                                logger.opt(colors=True).info(
-                                    "✨ <r>Step {counter}</r> <y>(agent: {agent_id})</y>",
-                                    counter=(counter + 1),
-                                    agent_id=agent_id,
-                                )
-                                response.live_log_state()
-                            counter += 1
-                        except Exception as e:
-                            if "error" in message and "last action failed with error" not in message:
-                                logger.error(f"Error in agent logs: {agent_id} {message}")
-                            elif agent_id in message and "agent_id" in message:
-                                logger.error(f"Error parsing AgentStatusResponse for message: {message}: {e}")
-                            else:
-                                logger.error(f"Error parsing agent logs for message: {message}: {e}")
-                            continue
 
-                        if response.is_completed():
+            def process_message(message: str) -> tuple[AgentCompletion | None, bool]:
+                """Process a websocket message. Returns (response, should_stop)."""
+                nonlocal counter
+                try:
+                    # try to json load
+                    dic = json.loads(message)
+                    response = None
+
+                    # output from validator
+                    if isinstance(dic, dict) and "validation" in dic:
+                        logger.opt(colors=True).info("<g>{message}</g>", message=dic["validation"])
+
+                    # termination message
+                    elif isinstance(dic, dict) and "status" in dic:
+                        if dic["status"] == "agent_stop":
+                            return (None, True)
+
+                    # actual step
+                    else:
+                        if isinstance(dic, dict):
+                            response = AgentCompletion.model_validate(dic)
+                        else:
+                            # Unexpected: log and skip
+                            logger.warning(f"Expected dict, got {type(dic).__name__}: {message[:200]}")
+                            return (None, False)
+                        if log:
+                            logger.opt(colors=True).info(
+                                "✨ <r>Step {counter}</r> <y>(agent: {agent_id})</y>",
+                                counter=(counter + 1),
+                                agent_id=agent_id,
+                            )
+                            response.live_log_state()
+                        counter += 1
+
+                    return (response, False)
+
+                except (json.JSONDecodeError, ValidationError, TypeError, ValueError) as e:
+                    if "error" in message and "last action failed with error" not in message:
+                        logger.error(f"Error in agent logs: {e} {agent_id} {message}")
+                    elif agent_id in message and "agent_id" in message:
+                        logger.error(f"Error parsing AgentStatusResponse for message: {message}: {e}")
+                    else:
+                        logger.error(f"Error parsing agent logs for message: {message}: {e}")
+                    return (None, False)
+
+            if RUNNING_IN_PYODIDE:
+                # Use JavaScript WebSocket API via Pyodide FFI
+                ws = js.WebSocket.new(wss_url)  # pyright: ignore [reportPossiblyUnboundVariable, reportUnknownMemberType, reportUnknownVariableType]
+                message_queue: asyncio.Queue[str | None] = asyncio.Queue()
+
+                # Create proxies for event handlers
+                def on_message(event: Any) -> None:
+                    message_queue.put_nowait(str(event.data))
+
+                def on_error(_event: Any) -> None:
+                    logger.error("WebSocket error occurred")
+
+                def on_close(_event: Any) -> None:
+                    message_queue.put_nowait(None)  # Signal end
+
+                on_message_proxy = create_proxy(on_message)  # pyright: ignore [reportPossiblyUnboundVariable, reportUnknownVariableType]
+                on_error_proxy = create_proxy(on_error)  # pyright: ignore [reportUnknownVariableType, reportPossiblyUnboundVariable]
+                on_close_proxy = create_proxy(on_close)  # pyright: ignore [reportUnknownVariableType, reportPossiblyUnboundVariable]
+
+                ws.addEventListener("message", on_message_proxy)  # pyright: ignore[reportUnknownMemberType]
+                ws.addEventListener("error", on_error_proxy)  # pyright: ignore[reportUnknownMemberType]
+                ws.addEventListener("close", on_close_proxy)  # pyright: ignore[reportUnknownMemberType]
+
+                # Wait for connection
+                while ws.readyState == 0:  # CONNECTING  # pyright: ignore [reportUnknownMemberType]
+                    await asyncio.sleep(0.1)
+
+                try:
+                    while True:
+                        message = await message_queue.get()
+                        if message is None:  # Connection closed
+                            break
+
+                        assert isinstance(message, str), f"Expected str, got {type(message)}"
+                        response, should_stop = process_message(message)
+
+                        if should_stop:
+                            return None
+
+                        if response is not None and response.is_completed():
                             logger.info(f"Agent {agent_id} completed in {counter} steps")
+                            return None
 
                 except ConnectionError as e:
                     logger.error(f"Connection error: {agent_id} {e}")
-                    return
+                    return None
                 except Exception as e:
                     logger.error(f"Error: {agent_id} {e} {traceback.format_exc()}")
-                    return
+                    return None
+                finally:
+                    try:
+                        ws.removeEventListener("message", on_message_proxy)  # pyright: ignore[reportUnknownMemberType]
+                        ws.removeEventListener("error", on_error_proxy)  # pyright: ignore[reportUnknownMemberType]
+                        ws.removeEventListener("close", on_close_proxy)  # pyright: ignore[reportUnknownMemberType]
+                    except Exception:
+                        pass
+                    on_message_proxy.destroy()  # pyright: ignore [reportUnknownMemberType]
+                    on_error_proxy.destroy()  # pyright: ignore [reportUnknownMemberType]
+                    on_close_proxy.destroy()  # pyright: ignore [reportUnknownMemberType]
+                    ws.close()  # pyright: ignore[reportUnknownMemberType]
+
+            else:
+                # Use native Python websockets library
+                async with client.connect(  # pyright: ignore[reportPossiblyUnboundVariable]
+                    uri=wss_url,
+                    open_timeout=30,
+                    ping_interval=5,
+                    ping_timeout=40,
+                    close_timeout=5,
+                    max_size=5 * (2**20),  # 5MB max size
+                ) as websocket:
+                    try:
+                        async for message in websocket:
+                            assert isinstance(message, str), f"Expected str, got {type(message)}"
+                            response, should_stop = process_message(message)
+
+                            if should_stop:
+                                return None
+
+                            if response is not None and response.is_completed():
+                                logger.info(f"Agent {agent_id} completed in {counter} steps")
+                                return None
+
+                    except ConnectionError as e:
+                        logger.error(f"Connection error: {agent_id} {e}")
+                        return None
+                    except Exception as e:
+                        logger.error(f"Error: {agent_id} {e} {traceback.format_exc()}")
+                        return None
 
         return await get_messages()
 
     async def watch_logs_and_wait(self, agent_id: str, session_id: str, log: bool = True) -> AgentStatusResponse:
         """
-        Asynchronously execute a task with the agent.
+        Execute a task with the agent and wait for completion.
 
-        This is currently a wrapper around the synchronous run method.
-        In future versions, this might be implemented as a true async operation.
+        This is an async method that watches logs and waits for the agent to complete.
 
         Args:
-            task (str): The task description for the agent to execute.
-            url (str | None): Optional starting URL for the task.
+            agent_id (str): The agent identifier.
+            session_id (str): The session identifier.
+            log (bool): Whether to log the agent steps.
 
         Returns:
-            AgentResponse: The response from the completed agent execution.
+            AgentStatusResponse: The response from the completed agent execution.
         """
         status = None
         try:
@@ -390,6 +520,53 @@ class AgentsClient(BaseClient):
             session_id=response.session_id,
         )
 
+    def workflow_code(self, agent_id: str, as_workflow: bool = True) -> AgentWorkflowCodeResponse:
+        """
+        Retrieves a script that reproduces the steps of the specified agent.
+
+        Queries the API using a validated agent ID.
+        The provided ID is confirmed (or obtained from the last response if needed), and the
+        resulting script is stored internally before being returned.
+
+        Args:
+            agent_id: Unique identifier of the agent to check.
+
+        Returns:
+            AgentWorkflowCodeResponse: The script that reproduces the steps of the specified agent
+
+        Raises:
+            ValueError: If no valid agent ID can be determined.
+        """
+        request = AgentWorkflowCodeRequest(as_workflow=as_workflow)
+        endpoint = AgentsClient._agent_workflow_endpoint(agent_id=agent_id).with_params(request)
+        response = self.request(endpoint)
+        return response
+
+    def workflow_create(self, agent_id: str) -> GetWorkflowResponse:
+        """
+        Creates a workflow that reproduces the steps of the specified agent.
+
+        Queries the API using a validated agent ID.
+        The provided ID is confirmed (or obtained from the last response if needed), and the
+        resulting script is stored internally before being returned.
+
+        Args:
+            agent_id: Unique identifier of the agent to check.
+
+        Returns:
+            GetWorkflowResponse: The workflow that reproduces the steps of the agent
+
+        Raises:
+            ValueError: If no valid agent ID can be determined.
+        """
+        script = self.workflow_code(agent_id, as_workflow=True)
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            filename = Path(tmp_dir) / "code.py"
+            with open(filename, "w") as f:
+                _ = f.write(script.python_script)
+
+            return self.root_client.workflows.create(workflow_path=str(filename))
+
     def status(self, agent_id: str) -> LegacyAgentStatusResponse:
         """
         Retrieves the status of the specified agent.
@@ -432,13 +609,13 @@ class AgentsClient(BaseClient):
 
     def replay(self, agent_id: str) -> MP4Replay:
         """
-        Downloads the replay for the specified agent in webp format.
+        Downloads the replay for the specified agent in mp4 format.
 
         ```python
         replay = agent.replay()
         ```
 
-        The replay is a webp file that can be displayed in a browser.
+        The replay is a mp4 file that can be displayed in a browser.
 
         ```python
         replay.show()
@@ -448,7 +625,7 @@ class AgentsClient(BaseClient):
             agent_id: The identifier of the agent to download the replay for.
 
         Returns:
-            WebpReplay: The replay file in webp format.
+            MP4Replay: The replay file in mp4 format.
         """
         endpoint = AgentsClient._agent_replay_endpoint(agent_id=agent_id)
         file_bytes = self._request_file(endpoint, file_type="mp4")
@@ -689,7 +866,7 @@ class RemoteAgent:
 
     The agent can be started, monitored, and controlled through various methods. It supports
     both synchronous and asynchronous execution modes, and can provide visual replays of
-    its actions in WEBP format.
+    its actions in MP4 format.
 
     Key Features:
     - Start and stop agent execution
@@ -707,6 +884,52 @@ class RemoteAgent:
     concurrent agents, create separate RemoteAgent instances.
     """
 
+    class AgentWorkflow:
+        def __init__(
+            self,
+            client: AgentsClient,
+            agent_id: str,
+        ):
+            self.client: AgentsClient = client
+            self.agent_id: str = agent_id
+
+        def code(self, as_workflow: bool = True) -> AgentWorkflowCodeResponse:
+            """
+            Retrieves a script that reproduces the steps of the specified agent.
+
+            Queries the API using a validated agent ID.
+            The provided ID is confirmed (or obtained from the last response if needed), and the
+            resulting script is stored internally before being returned.
+
+            Args:
+                as_workflow: Whether to return a full standalone workflow script or just the relevant steps
+
+            Returns:
+                AgentWorkflowCodeResponse: The script that reproduces the steps of the specified agent
+
+            Raises:
+                ValueError: If no valid agent ID can be determined.
+            """
+            return self.client.workflow_code(self.agent_id, as_workflow=as_workflow)
+
+        def create(self) -> RemoteWorkflow:
+            """
+            Creates a workflow that reproduces the steps of the specified agent.
+
+            Queries the API using a validated agent ID.
+            The provided ID is confirmed (or obtained from the last response if needed), and the
+            resulting script is stored internally before being returned.
+
+            Returns:
+                RemoteWorkflow: The workflow that reproduces the steps of the agent
+
+            Raises:
+                ValueError: If no valid agent ID can be determined.
+            """
+
+            workflow_resp = self.client.workflow_create(self.agent_id)
+            return RemoteWorkflow(workflow_id=workflow_resp.workflow_id, _client=self.client.root_client)
+
     @overload
     def __init__(
         self,
@@ -980,21 +1203,40 @@ class RemoteAgent:
         """
         return self.client.status(agent_id=self.agent_id)
 
+    @property
+    @track_usage("cloud.agent.workflow")
+    def workflow(self) -> AgentWorkflow:
+        """
+        Get the workflow from the completed steps of the agent.
+
+        ```python
+        agent.run(task="...")
+        workflow = agent.workflow
+        ```
+
+        Returns:
+            AgentWorkflow: The agent workflow that replicates the agent steps
+
+        Raises:
+            ValueError: If the agent hasn't been run yet (no agent_id available).
+        """
+        return RemoteAgent.AgentWorkflow(self.client, self.agent_id)
+
     @track_usage("cloud.agent.replay")
     def replay(self) -> MP4Replay:
         """
-        Get a replay of the agent's execution in WEBP format.
+        Get a replay of the agent's execution in MP4 format.
 
         This method downloads a visual replay of the agent's actions, which can be
         useful for debugging or understanding the agent's behavior.
 
         ```python
         replay = agent.replay()
-        replay.save(f"{agent.agent_id}_replay.webp")
+        replay.save(f"{agent.agent_id}_replay.mp4")
         ```
 
         Returns:
-            WebpReplay: The replay data in WEBP format.
+            MP4Replay: The replay data in MP4 format.
 
         Raises:
             ValueError: If the agent hasn't been run yet (no agent_id available).
