@@ -7,8 +7,8 @@ from collections.abc import Callable
 from typing import TYPE_CHECKING, Any, ClassVar, Unpack, final, overload
 
 import requests
-from loguru import logger
 from notte_core.ast import SecureScriptRunner
+from notte_core.common.logging import logger
 from notte_core.common.telemetry import track_usage
 from notte_core.errors.base import NotteBaseError
 from notte_core.utils.encryption import Encryption
@@ -21,6 +21,7 @@ from notte_sdk.types import (
     CreateWorkflowRunRequest,
     CreateWorkflowRunResponse,
     DeleteWorkflowResponse,
+    ForkWorkflowRequest,
     GetWorkflowRequest,
     GetWorkflowRequestDict,
     GetWorkflowResponse,
@@ -77,7 +78,8 @@ class WorkflowsClient(BaseClient):
 
     # Workflow endpoints
     CREATE_WORKFLOW = ""
-    UPDATE_WORKFLOW = "{workflow_id}"
+    FORK_WORKFLOW = "{workflow_id}/fork"
+    UPDATE_WORKFLOW = "{workflow_id}?restricted={restricted}"
     GET_WORKFLOW = "{workflow_id}"
     DELETE_WORKFLOW = "{workflow_id}"
     LIST_WORKFLOWS = ""
@@ -85,6 +87,7 @@ class WorkflowsClient(BaseClient):
     # RUN endpoints ...
     CREATE_WORKFLOW_RUN = "{workflow_id}/runs/create"
     START_WORKFLOW_RUN_WITHOUT_RUN_ID = "{workflow_id}/runs/start"
+    STOP_WORKFLOW_RUN = "{workflow_id}/runs/{run_id}"
     START_WORKFLOW_RUN = "{workflow_id}/runs/{run_id}"
     GET_WORKFLOW_RUN = "{workflow_id}/runs/{run_id}"
     LIST_WORKFLOW_RUNS = "{workflow_id}/runs/"
@@ -128,7 +131,7 @@ class WorkflowsClient(BaseClient):
         )
 
     @staticmethod
-    def _update_workflow_endpoint(workflow_id: str) -> NotteEndpoint[GetWorkflowResponse]:
+    def _update_workflow_endpoint(workflow_id: str, restricted: bool = True) -> NotteEndpoint[GetWorkflowResponse]:
         """
         Returns a NotteEndpoint configured for updating a workflow.
 
@@ -139,7 +142,7 @@ class WorkflowsClient(BaseClient):
             A NotteEndpoint with the POST method that expects a GetWorkflowResponse.
         """
         return NotteEndpoint(
-            path=WorkflowsClient.UPDATE_WORKFLOW.format(workflow_id=workflow_id),
+            path=WorkflowsClient.UPDATE_WORKFLOW.format(workflow_id=workflow_id, restricted=restricted),
             response=GetWorkflowResponse,
             method="POST",
         )
@@ -189,6 +192,17 @@ class WorkflowsClient(BaseClient):
             method="POST",
         )
 
+    @staticmethod
+    def _fork_workflow_endpoint(workflow_id: str) -> NotteEndpoint[GetWorkflowResponse]:
+        """
+        Returns a NotteEndpoint configured for forking a workflow.
+        """
+        return NotteEndpoint(
+            path=WorkflowsClient.FORK_WORKFLOW.format(workflow_id=workflow_id),
+            response=GetWorkflowResponse,
+            method="POST",
+        )
+
     @staticmethod
     def _start_workflow_run_endpoint(workflow_id: str, run_id: str) -> NotteEndpoint[WorkflowRunResponse]:
         """
@@ -211,6 +225,17 @@ class WorkflowsClient(BaseClient):
             method="POST",
         )
 
+    @staticmethod
+    def _stop_workflow_run_endpoint(workflow_id: str, run_id: str) -> NotteEndpoint[UpdateWorkflowRunResponse]:
+        """
+        Returns a NotteEndpoint configured for stopping a workflow run.
+        """
+        return NotteEndpoint(
+            path=WorkflowsClient.STOP_WORKFLOW_RUN.format(workflow_id=workflow_id, run_id=run_id),
+            response=UpdateWorkflowRunResponse,
+            method="DELETE",
+        )
+
     @staticmethod
     def _get_workflow_run_endpoint(workflow_id: str, run_id: str) -> NotteEndpoint[GetWorkflowRunResponse]:
         """
@@ -270,12 +295,25 @@ class WorkflowsClient(BaseClient):
             GetWorkflowResponse: The created workflow information.
         """
         request = CreateWorkflowRequest.model_validate(data)
-        endpoint = self._create_workflow_endpoint().with_file(request.workflow_path)
+        endpoint = self._create_workflow_endpoint().with_file(request.workflow_path).with_request(request)
         response = self.request(endpoint)
         return response
 
+    @track_usage("cloud.workflow.fork")
+    def fork(self, workflow_id: str) -> GetWorkflowResponse:
+        """
+        Fork a workflow.
+        """
+        request = ForkWorkflowRequest(workflow_id=workflow_id)
+        endpoint = self._fork_workflow_endpoint(workflow_id).with_request(request)
+        response = self.request(endpoint)
+        logger.info(f"[Workflow] {response.workflow_id} forked successfully from workflow_id={workflow_id}")
+        return response
+
     @track_usage("cloud.workflow.update")
-    def update(self, workflow_id: str, **data: Unpack[UpdateWorkflowRequestDict]) -> GetWorkflowResponse:
+    def update(
+        self, workflow_id: str, restricted: bool = True, **data: Unpack[UpdateWorkflowRequestDict]
+    ) -> GetWorkflowResponse:
         """
         Update an existing workflow.
 
@@ -287,7 +325,7 @@ class WorkflowsClient(BaseClient):
             GetWorkflowResponse: The updated workflow information.
         """
         request = UpdateWorkflowRequest.model_validate(data)
-        endpoint = self._update_workflow_endpoint(workflow_id).with_file(request.workflow_path)
+        endpoint = self._update_workflow_endpoint(workflow_id, restricted=restricted).with_file(request.workflow_path)
         if request.version is not None:
             endpoint = endpoint.with_params(GetWorkflowRequest(version=request.version))
         response = self.request(endpoint)
@@ -334,9 +372,12 @@ class WorkflowsClient(BaseClient):
         return self.request(self._list_workflows_endpoint().with_params(params))
 
     def create_run(self, workflow_id: str, local: bool = False) -> CreateWorkflowRunResponse:
-        request = CreateWorkflowRunRequest(workflow_id=workflow_id, local=local)
+        request = CreateWorkflowRunRequest(local=local)
         return self.request(self._create_workflow_run_endpoint(workflow_id).with_request(request))
 
+    def stop_run(self, workflow_id: str, run_id: str) -> UpdateWorkflowRunResponse:
+        return self.request(self._stop_workflow_run_endpoint(workflow_id, run_id))
+
     def get_run(self, workflow_id: str, run_id: str) -> GetWorkflowRunResponse:
         return self.request(self._get_workflow_run_endpoint(workflow_id, run_id))
 
@@ -469,7 +510,10 @@ class WorkflowsClient(BaseClient):
                         decoded = WorkflowsClient.decode_message(log_msg)
 
                         if message["type"] == "log":
-                            logger.opt(colors=True).info(decoded)
+                            try:
+                                logger.opt(colors=True).info(decoded)
+                            except Exception:
+                                logger.info(decoded)
                         elif message["type"] == "result":
                             result = log_msg
                         elif message["type"] == "session_start":
@@ -531,20 +575,42 @@ class RemoteWorkflow:
     ) -> None:
         if _client is None:
             raise ValueError("NotteClient is required")
-        if workflow_id is None:
-            response = _client.workflows.create(**data)
-            logger.info(f"[Workflow] {response.workflow_id} created successfully.")
-        else:
-            response = _client.workflows.get(workflow_id=workflow_id)
-            logger.info(f"[Workflow] {response.workflow_id} metadata retrieved successfully.")
         # init attributes
         self.client: WorkflowsClient = _client.workflows
         self.root_client: NotteClient = _client
-        self.response: GetWorkflowResponse | GetWorkflowWithLinkResponse = response
+        self._response: GetWorkflowResponse | GetWorkflowWithLinkResponse | None = None
+        if workflow_id is None:
+            self._response = _client.workflows.create(**data)
+            workflow_id = self._response.workflow_id
+            logger.info(f"[Workflow] {workflow_id} created successfully.")
+        self._workflow_id: str = workflow_id
         self._session_id: str | None = None
         self._workflow_run_id: str | None = None
         self.decryption_key: str | None = decryption_key
 
+    @property
+    def response(self) -> GetWorkflowResponse | GetWorkflowWithLinkResponse:
+        if self._response is not None:
+            return self._response
+        self._response = self.client.get(workflow_id=self._workflow_id)
+        logger.info(f"[Workflow] {self._response.workflow_id} metadata retrieved successfully.")
+        return self._response
+
+    def fork(self) -> "RemoteWorkflow":
+        """
+        Fork a shared workflow into your own private workflow.
+
+        ```python
+        workflow = notte.Workflow("<user-shared-workflow-id>")
+        forked_workflow = workflow.fork()
+        forked_workflow.run()
+        ```
+
+        The forked workflow is only accessible to you and you can update it as you want.
+        """
+        fork_response = self.client.fork(workflow_id=self._workflow_id)
+        return RemoteWorkflow(workflow_id=fork_response.workflow_id, _client=self.root_client)
+
     @property
     def workflow_id(self) -> str:
         return self.response.workflow_id
@@ -570,7 +636,7 @@ class RemoteWorkflow:
             )
         return self.root_client.sessions.replay(session_id=self._session_id)
 
-    def update(self, workflow_path: str, version: str | None = None) -> None:
+    def update(self, workflow_path: str, version: str | None = None, restricted: bool = True) -> None:
         """
         Update the workflow with a a new code version.
 
@@ -581,8 +647,8 @@ class RemoteWorkflow:
 
         If you set a version, only that version will be updated.
         """
-        self.response = self.client.update(
-            workflow_id=self.response.workflow_id, workflow_path=workflow_path, version=version
+        self._response = self.client.update(
+            workflow_id=self.response.workflow_id, workflow_path=workflow_path, version=version, restricted=restricted
         )
         logger.info(
             f"[Workflow] {self.response.workflow_id} updated successfully to version {self.response.latest_version}."
@@ -600,17 +666,21 @@ class RemoteWorkflow:
         _ = self.client.delete(workflow_id=self.response.workflow_id)
         logger.info(f"[Workflow] {self.response.workflow_id} deleted successfully.")
 
-    def get_url(self, version: str | None = None) -> str:
+    def get_url(self, version: str | None = None, decryption_key: str | None = None) -> str:
         if not isinstance(self.response, GetWorkflowWithLinkResponse) or version != self.response.latest_version:
-            self.response = self.client.get(workflow_id=self.response.workflow_id, version=version)
-        url = self.response.url
+            self._response = self.client.get(workflow_id=self.response.workflow_id, version=version)
+            url = self._response.url
+        else:
+            url = self.response.url
+
+        decryption_key = decryption_key or self.decryption_key
         decrypted: bool = url.startswith("https://") or url.startswith("http://")
         if not decrypted:
-            if self.decryption_key is None:
+            if decryption_key is None:
                 raise ValueError(
                     "Decryption key is required to decrypt the workflow download url. Set the `notte.Workflow(workflow_id='<your-workflow-id>', decryption_key='<your-key>')` when creating the workflow."
                 )
-            encryption = Encryption(root_key=self.decryption_key)
+            encryption = Encryption(root_key=decryption_key)
             url = encryption.decrypt(url)
             decrypted = url.startswith("https://") or url.startswith("http://")
             if not decrypted:
@@ -620,20 +690,20 @@ class RemoteWorkflow:
             logger.info("🔐 Successfully decrypted workflow download url")
         return url
 
-    def download(self, workflow_path: str | None, version: str | None = None) -> str:
+    def download(self, workflow_path: str | None, version: str | None = None, decryption_key: str | None = None) -> str:
         """
         Download the workflow code from the notte console as a python file.
 
         ```python
         workflow = notte.Workflow("<your-workflow-id>")
-        workflow.download(workflow_path="<path-to-your-workflow.py>")
+        workflow.download(workflow_path="<path-to-your-workflow.py>", decryption_key="<your-key>")
         ```
 
         """
         if workflow_path is not None and not workflow_path.endswith(".py"):
             raise ValueError(f"Workflow path must end with .py, got '{workflow_path}'")
 
-        file_url = self.get_url(version=version)
+        file_url = self.get_url(version=version, decryption_key=decryption_key)
         try:
             response = requests.get(file_url, timeout=self.client.DEFAULT_REQUEST_TIMEOUT_SECONDS)
             response.raise_for_status()
@@ -667,12 +737,12 @@ class RemoteWorkflow:
 
         ```python
         workflow = notte.Workflow("<your-workflow-id>")
-        workflow.run(variable1="value1", variable2="value2", local=True)
+        workflow.run(variable1="value1", variable2="value2")
         ```
 
         > Make sure that the correct variables are provided based on the python file previously uploaded. Otherwise, the workflow will fail.
 
-        You can use `local=True` to run the workflow locally.
+        You can use `local=True` to run the workflow locally if you have access to the decryption key.
 
         """
         # first create the run on DB
@@ -735,5 +805,23 @@ class RemoteWorkflow:
         self._session_id = res.session_id
         return res
 
+    def stop_run(self, run_id: str) -> UpdateWorkflowRunResponse:
+        """
+        Manually stop a workflow run by its ID.
+
+        """
+        return self.client.stop_run(workflow_id=self.workflow_id, run_id=run_id)
+
+    def get_run(self, run_id: str) -> GetWorkflowRunResponse:
+        """
+        Get a workflow run by its ID.
+
+        """
+        return self.client.get_run(workflow_id=self.workflow_id, run_id=run_id)
+
     def get_curl(self, **variables: Any) -> str:
+        """
+        Convert the workflow/run to a curl request.
+
+        """
         return self.client.get_curl(workflow_id=self.workflow_id, **variables)
